<?php
/**
 * Password Generator.
 *
 * This file includes various password generation algorithms.
 *
 * @author Alexandros Vellis <avel@noc.uoa.gr>
 * @author Stefanos Stamatis <stef@noc.uoa.gr>
 * @package components
 * @subpackage password_generator
 * @version $Id: password_generator.inc.php 2627 2008-06-05 08:01:34Z avel $
 */

/**
 * Katsika Generate Password wrapper.
 * This will reflect the current generator that is in use, or combine different
 * generators.
 *
 * @return string
 */
function uoa_generate_password() {
    // 1) generate an easy-to-say string
    $easy = password_generate_really_easy_to_say();
    // 2) generate a totally random part
    $hard_len = rand(2,3);
    $hard = password_generate_random($hard_len, false, true, false);
    // 3) Concatenate them
    if($flag = rand(0,1)) {
        return $easy.$hard;
    } else {
        return $hard.$easy;
    }
}

/**
 * Nice function that generates an easy-to-pronounce string.
 * @author stef
 */
function password_generate_really_easy_to_say() {
    $makepass="";
    $syllables="er,in,tia,wol,fe,pre,vet,jo,nes,al,len,son,cha,ir,ler,bo,ok,tio,nar,sim,ple,bla,ten,toe,cho,co,lat,spe,ak,er,po,c
o,lor,pen,cil,li,ght,wh,at,the,he,ck,is,mam,bo,no,fi,ve,any,way,pol,iti,cs,ra,dio,sou,rce,sea,rch,pa,per,com,bo,sp,eak,st,fi,rst,
gr,oup,boy,ea,gle,tr,ail,bi,ble,brb,pri,dee,kay,en,be,se";
    $syllable_array=explode(",", $syllables);
    srand((double)microtime()*1000000);
    while(strlen($makepass)<8) {
        if (rand()%10 == 1) {
            $makepass .= sprintf("%0.0f",(rand()%50)+1);
        } else {
            $makepass .= sprintf("%s",$syllable_array[rand()%62]);
        }
    }
    return(substr(str_replace("\n", '', $makepass),0,8));
}

/**
 * Gen_Password -- Easy-to-Say passwords generator
 *
 * @param $p string Predefined part of password ("" upto 10 chars but less than $l)
 * @param $l int - password length (1-49, def: 8)
 * @param $f int Fuzzy (0-10, def: 3)
 * @return string
 * @author Ell Gree <ellgree@gmx.net>
 */
function password_generate_easy_to_say($p="", $l=8, $f=4) {
  $d=array('a'=>'ntrsldicmzp','b'=>'euloayribsj','c'=>'oheaktirulc',
       'd'=>'eiorasydlun','e'=>'nrdsaltevcm','f'=>'ioreafltuyc',
       'g'=>'aeohrilunsg','h'=>'eiaotruykms','i'=>'ntscmledorg',
       'j'=>'ueoairhjklm','k'=>'eiyonashlus','l'=>'eoiyaldsfut',
       'm'=>'eaoipsuybmn','n'=>'goeditscayl','o'=>'fnrzmwtovls',
       'p'=>'earolipuths','q'=>'uuuuaecdfok','r'=>'eoiastydgnm',
       's'=>'eothisakpuc','t'=>'hoeiarzsuly','u'=>'trsnlpgecim',
       'v'=>'eiaosnykrlu','w'=>'aiheonrsldw','x'=>'ptciaeuohnq',
       'y'=>'oesitabpmwc','z'=>'eaiozlryhmt');
  $a=range('a','z');
  $l%=50; $f%=11;
  $p=strtolower(ereg_replace("[^a-zA-Z]","",substr($p,0,$l-1))) or
  $p=$a[rand(0,sizeof($a)-1)];
  while(strlen($p)<$l) {
    $ff = $f;
    while(substr_count($p,substr($p,strlen($p)-1,1).
      ($k=substr($d[substr($p,strlen($p)-1,1)],rand(0,$ff%11),1))))
    if(++$ff>10) break;
    $p.=$k;
  }
  return $p;
}


/**
 * Generate password based on given params.
 *
 * Originally written by:
 * hitech-password.php - a PHP Message board script
 * (c) Hitech Scripts 2003
 * For more information, visit http://www.hitech-scripts.com
 * @return string
 */
function password_generate_random($length = 8, $use_mix = false, $use_num = true, $use_let = true)    {
    $allowable_characters = "ABCDEFGHJKLMNPQRSTUVWXYZabcdefghjklmnpqrstuvwxyz23456789";
   
    if ($use_mix && $use_num && $use_let)  {
      $ps_st = 0;
      $ps_len = strlen($allowable_characters);
    }
     
    if (!$use_mix && $use_num && $use_let)  {
      $ps_st = 24;
      $ps_len = strlen($allowable_characters);
    }
 
    if ($use_mix && !$use_num && $use_let)  {
      $ps_st = 0;
      $ps_len = 47;
    }
 
    if ($use_mix && $use_num && !$use_let)  {
      $ps_st = 48;
      $ps_len = strlen($allowable_characters);
    }
 
    if (!$use_mix && !$use_num && $use_let)  {
      $ps_st = 24;
      $ps_len = 47;
    }
 
    if (!$use_mix && $use_num && !$use_let)  {
      $ps_st = 48;
      $ps_len = strlen($allowable_characters);
    }
 
    if ($use_mix && !$use_num && !$use_let)  {
      $ps_st = 0;
      $ps_len = 1;
    }
    
    if (!$use_mix && !$use_num && !$use_let)  {
      $ps_st = 0;
      $ps_len = 1;
    }
    
    mt_srand((double)microtime()*1000000);
 
    $pass = "";
 
    for($i = 0; $i < $length; $i++) {
        $pass .= $allowable_characters[mt_rand($ps_st, $ps_len - 1)];
    }
    
    return $pass;
}

/**
 * Totally random string or something. (probably useless and obsolete).
 */
function password_generate_totally_random($length=16){
    mt_srand((double)microtime()*1000000);
    $newstring="";

    if($length>0){
        while(strlen($newstring)<$length){
            switch(mt_rand(1,3)){
                case 1: $newstring.=chr(mt_rand(48,57)); break;  // 0-9
                case 2: $newstring.=chr(mt_rand(65,90)); break;  // A-Z
                case 3: $newstring.=chr(mt_rand(97,122)); break; // a-z
            }
        }
    }
    return $newstring;
}

