<?php
/**
 * Support functions for password strength checking.
 *
 * @package components
 * @subpackage password_strength_check
 * @version $Id: functions.inc.php 5593 2012-03-13 12:42:25Z avel $
 * @license http://opensource.org/licenses/gpl-license.php GNU Public License
 */

/**
 * LCS functions ripped from the php manual.
 *
 * @author georgesk at hotmail dot com
 * @see http://gr2.php.net/manual/en/function.similar-text.php
 */
function LCS_Length($s1, $s2)
{
  $m = strlen($s1);
  $n = strlen($s2);

  //this table will be used to compute the LCS-Length, only 128 chars per string are considered
  $LCS_Length_Table = array(array(128),array(128)); 
  
  //reset the 2 cols in the table
  for($i=1; $i <= $m; $i++) $LCS_Length_Table[$i][0]=0;
  for($j=0; $j <= $n; $j++) $LCS_Length_Table[0][$j]=0;

  for ($i=1; $i <= $m; $i++) {
   for ($j=1; $j <= $n; $j++) {
     if ($s1[$i-1]==$s2[$j-1])
       $LCS_Length_Table[$i][$j] = $LCS_Length_Table[$i-1][$j-1] + 1;
     else if ($LCS_Length_Table[$i-1][$j] >= $LCS_Length_Table[$i][$j-1])
       $LCS_Length_Table[$i][$j] = $LCS_Length_Table[$i-1][$j];
     else
       $LCS_Length_Table[$i][$j] = $LCS_Length_Table[$i][$j-1];
   }
  }
  return $LCS_Length_Table[$m][$n];
}

/**
 * LCS functions ripped from the php manual.
 */
function str_lcsfix($s)
{
  $s = str_replace(" ","",$s);
  $s = preg_replace("/[éèêëËÊÉÈ]/","e", $s);
  $s = preg_replace("/[àáâãäåÄÅÃÂÁÀ]/","a", $s);
  $s = preg_replace("/[ìíîïÏÎÍÌ]/","i", $s);
  $s = preg_replace("/[òóôõöÖÕÔÓ]/","o", $s);
  $s = preg_replace("/[ÜÛÚÙùúûü]/","u", $s);
  $s = preg_replace("/[Ç]/","c", $s);
  return $s;
}
  
/**
 * LCS functions ripped from the php manual.
 */
function get_lcs($s1, $s2)
{
  //ok, now replace all spaces with nothing
  $s1 = strtolower(str_lcsfix($s1));
  $s2 = strtolower(str_lcsfix($s2));

  $lcs = LCS_Length($s1,$s2); //longest common sub sequence

  $ms = (strlen($s1) + strlen($s2)) / 2;

  return (($lcs*100)/$ms);
}

